# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from wds_client.models.record_query_response import RecordQueryResponse

class TestRecordQueryResponse(unittest.TestCase):
    """RecordQueryResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> RecordQueryResponse:
        """Test RecordQueryResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `RecordQueryResponse`
        """
        model = RecordQueryResponse()
        if include_optional:
            return RecordQueryResponse(
                search_request = wds_client.models.search_request.SearchRequest(
                    offset = 0, 
                    limit = 0, 
                    sort = 'ASC', 
                    sort_attribute = '', 
                    filter = wds_client.models.search_filter.SearchFilter(
                        ids = [
                            ''
                            ], 
                        query = 'sample_id:12345', ), ),
                total_records = 56,
                records = [
                    wds_client.models.record_response.RecordResponse(
                        id = '', 
                        type = '', 
                        attributes = {
  "stringAttr": "string",
  "numericAttr": 123,
  "booleanAttr": true,
  "relationAttr": "terra-wds:/target-type/target-id",
  "fileAttr": "https://account_name.blob.core.windows.net/container-1/blob1",
  "arrayString": ["green", "red"],
  "arrayBoolean": [true, false],
  "arrayNumber": [12821.112, 0.12121211, 11],
  "arrayDate": ["2022-11-03"],
  "arrayDateTime": ["2022-11-03T04:36:20"],
  "arrayRelation": ["terra-wds:/target-type/target-id-1", "terra-wds:/target-type/target-id-2"],
  "arrayFile": ["drs://drs.example.org/file_id_1", "https://account_name.blob.core.windows.net/container-2/blob2"]
}
, )
                    ]
            )
        else:
            return RecordQueryResponse(
                search_request = wds_client.models.search_request.SearchRequest(
                    offset = 0, 
                    limit = 0, 
                    sort = 'ASC', 
                    sort_attribute = '', 
                    filter = wds_client.models.search_filter.SearchFilter(
                        ids = [
                            ''
                            ], 
                        query = 'sample_id:12345', ), ),
                total_records = 56,
                records = [
                    wds_client.models.record_response.RecordResponse(
                        id = '', 
                        type = '', 
                        attributes = {
  "stringAttr": "string",
  "numericAttr": 123,
  "booleanAttr": true,
  "relationAttr": "terra-wds:/target-type/target-id",
  "fileAttr": "https://account_name.blob.core.windows.net/container-1/blob1",
  "arrayString": ["green", "red"],
  "arrayBoolean": [true, false],
  "arrayNumber": [12821.112, 0.12121211, 11],
  "arrayDate": ["2022-11-03"],
  "arrayDateTime": ["2022-11-03T04:36:20"],
  "arrayRelation": ["terra-wds:/target-type/target-id-1", "terra-wds:/target-type/target-id-2"],
  "arrayFile": ["drs://drs.example.org/file_id_1", "https://account_name.blob.core.windows.net/container-2/blob2"]
}
, )
                    ],
        )
        """

    def testRecordQueryResponse(self):
        """Test RecordQueryResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AttributeDataType(str, Enum):
    """
    Datatype of attribute. The enum of data types is in flux and will change. Please comment at https://docs.google.com/document/d/1d352ZoN5kEYWPjy0NqqWGxdf7HEu5VEdrLmiAv7dMmQ/edit#heading=h.naxag0augkgf. 
    """

    """
    allowed enum values
    """
    BOOLEAN = 'BOOLEAN'
    NUMBER = 'NUMBER'
    DATE = 'DATE'
    DATE_TIME = 'DATE_TIME'
    STRING = 'STRING'
    JSON = 'JSON'
    RELATION = 'RELATION'
    FILE = 'FILE'
    ARRAY_OF_BOOLEAN = 'ARRAY_OF_BOOLEAN'
    ARRAY_OF_STRING = 'ARRAY_OF_STRING'
    ARRAY_OF_NUMBER = 'ARRAY_OF_NUMBER'
    ARRAY_OF_DATE = 'ARRAY_OF_DATE'
    ARRAY_OF_DATE_TIME = 'ARRAY_OF_DATE_TIME'
    ARRAY_OF_RELATION = 'ARRAY_OF_RELATION'
    ARRAY_OF_FILE = 'ARRAY_OF_FILE'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AttributeDataType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from wds_client.api.general_wds_information_api import GeneralWDSInformationApi


class TestGeneralWDSInformationApi(unittest.TestCase):
    """GeneralWDSInformationApi unit test stubs"""

    def setUp(self) -> None:
        self.api = GeneralWDSInformationApi()

    def tearDown(self) -> None:
        pass

    def test_status_get(self) -> None:
        """Test case for status_get

        Gets health status for WDS -- generated via Spring Boot Actuator (see https://docs.spring.io/spring-boot/docs/current/actuator-api/htmlsingle/#health for details)
        """
        pass

    def test_version_get(self) -> None:
        """Test case for version_get

        Gets related git and build version info for WDS -- generated via Spring Boot Actuator (see https://docs.spring.io/spring-boot/docs/current/actuator-api/htmlsingle/#info for details)
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class SearchSortDirection(str, Enum):
    """
    Sort direction (descending or ascending) 
    """

    """
    allowed enum values
    """
    ASC = 'ASC'
    DESC = 'DESC'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchSortDirection from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from wds_client.api.collection_api import CollectionApi


class TestCollectionApi(unittest.TestCase):
    """CollectionApi unit test stubs"""

    def setUp(self) -> None:
        self.api = CollectionApi()

    def tearDown(self) -> None:
        pass

    def test_create_collection_v1(self) -> None:
        """Test case for create_collection_v1

        Create a collection in this workspace.
        """
        pass

    def test_delete_collection_v1(self) -> None:
        """Test case for delete_collection_v1

        Delete the specified collection.
        """
        pass

    def test_get_collection_v1(self) -> None:
        """Test case for get_collection_v1

        Retrieve a single collection.
        """
        pass

    def test_list_collections_v1(self) -> None:
        """Test case for list_collections_v1

        List all collections in this workspace.
        """
        pass

    def test_update_collection_v1(self) -> None:
        """Test case for update_collection_v1

        Update the specified collection.
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBytes, StrictStr
from typing import List, Optional, Union
from typing_extensions import Annotated
from wds_client.models.batch_operation import BatchOperation
from wds_client.models.batch_response import BatchResponse
from wds_client.models.record_query_response import RecordQueryResponse
from wds_client.models.record_request import RecordRequest
from wds_client.models.record_response import RecordResponse
from wds_client.models.search_request import SearchRequest
from wds_client.models.tsv_upload_response import TsvUploadResponse

from wds_client.api_client import ApiClient, RequestSerialized
from wds_client.api_response import ApiResponse
from wds_client.rest import RESTResponseType


class RecordsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def batch_write_records(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        batch_operation: Annotated[List[BatchOperation], Field(description="A list of batch operations to perform on records")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> BatchResponse:
        """Batch write records

        Perform a batch of upsert / delete operations on multiple records

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param batch_operation: A list of batch operations to perform on records (required)
        :type batch_operation: List[BatchOperation]
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._batch_write_records_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            batch_operation=batch_operation,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BatchResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def batch_write_records_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        batch_operation: Annotated[List[BatchOperation], Field(description="A list of batch operations to perform on records")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[BatchResponse]:
        """Batch write records

        Perform a batch of upsert / delete operations on multiple records

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param batch_operation: A list of batch operations to perform on records (required)
        :type batch_operation: List[BatchOperation]
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._batch_write_records_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            batch_operation=batch_operation,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BatchResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def batch_write_records_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        batch_operation: Annotated[List[BatchOperation], Field(description="A list of batch operations to perform on records")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Batch write records

        Perform a batch of upsert / delete operations on multiple records

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param batch_operation: A list of batch operations to perform on records (required)
        :type batch_operation: List[BatchOperation]
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._batch_write_records_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            batch_operation=batch_operation,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BatchResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _batch_write_records_serialize(
        self,
        instanceid,
        v,
        type,
        batch_operation,
        primary_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'BatchOperation': '',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        # process the query parameters
        if primary_key is not None:
            
            _query_params.append(('primaryKey', primary_key))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if batch_operation is not None:
            _body_params = batch_operation


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/{instanceid}/batch/{v}/{type}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def create_or_replace_record(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        record_request: Annotated[RecordRequest, Field(description="A record's attributes to upload")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RecordResponse:
        """Create or replace record

        Creates or replaces the record using the specified type and id. If the record already exists, its entire set of attributes will be overwritten by the attributes in the request body. TODO: add a query parameter to allow/disallow overwriting existing records? 

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param record_request: A record's attributes to upload (required)
        :type record_request: RecordRequest
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_or_replace_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            record_request=record_request,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '201': "RecordResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_or_replace_record_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        record_request: Annotated[RecordRequest, Field(description="A record's attributes to upload")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RecordResponse]:
        """Create or replace record

        Creates or replaces the record using the specified type and id. If the record already exists, its entire set of attributes will be overwritten by the attributes in the request body. TODO: add a query parameter to allow/disallow overwriting existing records? 

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param record_request: A record's attributes to upload (required)
        :type record_request: RecordRequest
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_or_replace_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            record_request=record_request,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '201': "RecordResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_or_replace_record_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        record_request: Annotated[RecordRequest, Field(description="A record's attributes to upload")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create or replace record

        Creates or replaces the record using the specified type and id. If the record already exists, its entire set of attributes will be overwritten by the attributes in the request body. TODO: add a query parameter to allow/disallow overwriting existing records? 

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param record_request: A record's attributes to upload (required)
        :type record_request: RecordRequest
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_or_replace_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            record_request=record_request,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '201': "RecordResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_or_replace_record_serialize(
        self,
        instanceid,
        v,
        type,
        id,
        record_request,
        primary_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        if primary_key is not None:
            
            _query_params.append(('primaryKey', primary_key))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if record_request is not None:
            _body_params = record_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/{instanceid}/records/{v}/{type}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_record(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete record

        Deletes the record at the specified type and id.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '404': "ErrorResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_record_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete record

        Deletes the record at the specified type and id.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '404': "ErrorResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_record_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete record

        Deletes the record at the specified type and id.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '404': "ErrorResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_record_serialize(
        self,
        instanceid,
        v,
        type,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/{instanceid}/records/{v}/{type}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_record(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RecordResponse:
        """Get record

        Retrieves a single record by its type and id

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_record_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RecordResponse]:
        """Get record

        Retrieves a single record by its type and id

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_record_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get record

        Retrieves a single record by its type and id

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_record_serialize(
        self,
        instanceid,
        v,
        type,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/{instanceid}/records/{v}/{type}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_records_as_tsv(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> bytearray:
        """Retrieve all records in record type as tsv.

        Streams all records in a record type to a tsv format.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_records_as_tsv_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bytearray",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_records_as_tsv_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[bytearray]:
        """Retrieve all records in record type as tsv.

        Streams all records in a record type to a tsv format.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_records_as_tsv_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bytearray",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_records_as_tsv_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve all records in record type as tsv.

        Streams all records in a record type to a tsv format.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_records_as_tsv_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bytearray",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_records_as_tsv_serialize(
        self,
        instanceid,
        v,
        type,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'text/tab-separated-values', 
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/{instanceid}/tsv/{v}/{type}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def query_records(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        search_request: Annotated[SearchRequest, Field(description="A paginated search request")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RecordQueryResponse:
        """Query records

        Paginated list of records matching the criteria supplied in the request body

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param search_request: A paginated search request (required)
        :type search_request: SearchRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._query_records_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            search_request=search_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordQueryResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def query_records_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        search_request: Annotated[SearchRequest, Field(description="A paginated search request")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RecordQueryResponse]:
        """Query records

        Paginated list of records matching the criteria supplied in the request body

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param search_request: A paginated search request (required)
        :type search_request: SearchRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._query_records_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            search_request=search_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordQueryResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def query_records_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        search_request: Annotated[SearchRequest, Field(description="A paginated search request")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Query records

        Paginated list of records matching the criteria supplied in the request body

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param search_request: A paginated search request (required)
        :type search_request: SearchRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._query_records_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            search_request=search_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordQueryResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _query_records_serialize(
        self,
        instanceid,
        v,
        type,
        search_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if search_request is not None:
            _body_params = search_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/{instanceid}/search/{v}/{type}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_record(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        record_request: Annotated[RecordRequest, Field(description="A record's attributes to upload")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RecordResponse:
        """Update record

        Updates the record of the specified type and id. Any attributes included in the request body will be created or overwritten. Attributes not included in the request body will be untouched in the database. No attributes will be deleted. To delete attributes, use the PUT api instead. 

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param record_request: A record's attributes to upload (required)
        :type record_request: RecordRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            record_request=record_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '404': "ErrorResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_record_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        record_request: Annotated[RecordRequest, Field(description="A record's attributes to upload")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RecordResponse]:
        """Update record

        Updates the record of the specified type and id. Any attributes included in the request body will be created or overwritten. Attributes not included in the request body will be untouched in the database. No attributes will be deleted. To delete attributes, use the PUT api instead. 

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param record_request: A record's attributes to upload (required)
        :type record_request: RecordRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            record_request=record_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '404': "ErrorResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_record_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        id: Annotated[StrictStr, Field(description="Record id")],
        record_request: Annotated[RecordRequest, Field(description="A record's attributes to upload")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update record

        Updates the record of the specified type and id. Any attributes included in the request body will be created or overwritten. Attributes not included in the request body will be untouched in the database. No attributes will be deleted. To delete attributes, use the PUT api instead. 

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param id: Record id (required)
        :type id: str
        :param record_request: A record's attributes to upload (required)
        :type record_request: RecordRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_record_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            id=id,
            record_request=record_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RecordResponse",
            '404': "ErrorResponse",
            '400': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_record_serialize(
        self,
        instanceid,
        v,
        type,
        id,
        record_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if record_request is not None:
            _body_params = record_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/{instanceid}/records/{v}/{type}/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def upload_tsv(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        records: Annotated[Union[StrictBytes, StrictStr], Field(description="A valid TSV import file")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TsvUploadResponse:
        """Import records to a record type from a tsv file

        Upload a tsv to modify or create records in a record type.  This operation will insert or update records.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param records: A valid TSV import file (required)
        :type records: bytearray
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._upload_tsv_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            records=records,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TsvUploadResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def upload_tsv_with_http_info(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        records: Annotated[Union[StrictBytes, StrictStr], Field(description="A valid TSV import file")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TsvUploadResponse]:
        """Import records to a record type from a tsv file

        Upload a tsv to modify or create records in a record type.  This operation will insert or update records.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param records: A valid TSV import file (required)
        :type records: bytearray
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._upload_tsv_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            records=records,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TsvUploadResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def upload_tsv_without_preload_content(
        self,
        instanceid: Annotated[StrictStr, Field(description="WDS instance id; by convention equal to workspace id")],
        v: Annotated[StrictStr, Field(description="API version")],
        type: Annotated[StrictStr, Field(description="Record type")],
        records: Annotated[Union[StrictBytes, StrictStr], Field(description="A valid TSV import file")],
        primary_key: Annotated[Optional[StrictStr], Field(description="the column to uniquely identify a record")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Import records to a record type from a tsv file

        Upload a tsv to modify or create records in a record type.  This operation will insert or update records.

        :param instanceid: WDS instance id; by convention equal to workspace id (required)
        :type instanceid: str
        :param v: API version (required)
        :type v: str
        :param type: Record type (required)
        :type type: str
        :param records: A valid TSV import file (required)
        :type records: bytearray
        :param primary_key: the column to uniquely identify a record
        :type primary_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._upload_tsv_serialize(
            instanceid=instanceid,
            v=v,
            type=type,
            records=records,
            primary_key=primary_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TsvUploadResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _upload_tsv_serialize(
        self,
        instanceid,
        v,
        type,
        records,
        primary_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instanceid is not None:
            _path_params['instanceid'] = instanceid
        if v is not None:
            _path_params['v'] = v
        if type is not None:
            _path_params['type'] = type
        # process the query parameters
        if primary_key is not None:
            
            _query_params.append(('primaryKey', primary_key))
            
        # process the header parameters
        # process the form parameters
        if records is not None:
            _files['records'] = records
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/{instanceid}/tsv/{v}/{type}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

# flake8: noqa

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "0.38.30"

# import apis into sdk package
from wds_client.api.capabilities_api import CapabilitiesApi
from wds_client.api.collection_api import CollectionApi
from wds_client.api.general_wds_information_api import GeneralWDSInformationApi
from wds_client.api.import_api import ImportApi
from wds_client.api.job_api import JobApi
from wds_client.api.record_api import RecordApi
from wds_client.api.records_api import RecordsApi
from wds_client.api.schema_api import SchemaApi
from wds_client.api.workspace_api import WorkspaceApi

# import ApiClient
from wds_client.api_response import ApiResponse
from wds_client.api_client import ApiClient
from wds_client.configuration import Configuration
from wds_client.exceptions import OpenApiException
from wds_client.exceptions import ApiTypeError
from wds_client.exceptions import ApiValueError
from wds_client.exceptions import ApiKeyError
from wds_client.exceptions import ApiAttributeError
from wds_client.exceptions import ApiException

# import models into sdk package
from wds_client.models.app import App
from wds_client.models.attribute_data_type import AttributeDataType
from wds_client.models.attribute_schema import AttributeSchema
from wds_client.models.attribute_schema_update import AttributeSchemaUpdate
from wds_client.models.batch_operation import BatchOperation
from wds_client.models.batch_record_request import BatchRecordRequest
from wds_client.models.batch_response import BatchResponse
from wds_client.models.build import Build
from wds_client.models.capabilities import Capabilities
from wds_client.models.collection import Collection
from wds_client.models.collection_request import CollectionRequest
from wds_client.models.commit import Commit
from wds_client.models.delete_records_request import DeleteRecordsRequest
from wds_client.models.delete_records_response import DeleteRecordsResponse
from wds_client.models.error_response import ErrorResponse
from wds_client.models.generic_job import GenericJob
from wds_client.models.git import Git
from wds_client.models.import_request import ImportRequest
from wds_client.models.job_v1 import JobV1
from wds_client.models.record_query_response import RecordQueryResponse
from wds_client.models.record_request import RecordRequest
from wds_client.models.record_response import RecordResponse
from wds_client.models.record_type_schema import RecordTypeSchema
from wds_client.models.search_filter import SearchFilter
from wds_client.models.search_request import SearchRequest
from wds_client.models.search_sort_direction import SearchSortDirection
from wds_client.models.status_response import StatusResponse
from wds_client.models.tsv_upload_response import TsvUploadResponse
from wds_client.models.version_response import VersionResponse
from wds_client.models.workspace_init import WorkspaceInit
from wds_client.models.workspace_init_clone import WorkspaceInitClone
